package	jp.co.sds_corp.streaming;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Vector;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.nanchart.graph.DefaultStorage;
import jp.co.sds_corp.nanchart.graph.Graph2D;
import jp.co.sds_corp.nanchart.graph.StairGraph;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.ratemixer.fx.FxRateKey;
import jp.co.sds_corp.transfer.IReceiveListener;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.ServiceLoader;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.pooh.custom.IRubberBand;
import jp.co.sds_corp.util.pooh.custom.RubberBand;
import jp.co.sds_corp.util.pooh.simple.SimplePanel;


public class ChartPanel extends SimplePanel
	implements IReceiveListener, IRubberBand
{
	ServiceLoader	recv;
	private RubberBand	    rubb;
	private Vector		    vecHist = new Vector();
	private int				point = 2;
	private DecimalFormat	df;
	String	ccy1, ccy2, term;
	int		nPrec;
	MasterCache master;
	CcyPair	pair;
	FxRateKey key;

	public ChartPanel() {
	}
	public boolean createPane() {
		rubb = new RubberBand(this);
		String pattern = "0.0000000000".substring( 0, point+2 );
		df = new DecimalFormat(pattern);
		return true;
	}

	public void initPanel( String ccy1, String ccy2, String term, String prec ) {
		this.ccy1 = ccy1;
		this.ccy2 = ccy2;
		this.term = term;
		this.nPrec = Integer.valueOf( prec ).intValue();
		master = MasterCache.getInstance();
		try {
			pair = master.getCcyPair( ccy1, ccy2 );
		} catch ( Exception e ) {
		}
		setTitle( ccy1 + "/" + ccy2 + " " + term );
		try {
			key = new FxRateKey( pair, term );
			recv = ServiceLoader.getInstance();
			recv.addReceiveListener( key, this );
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

	public void doRubberBand(Rectangle rect){
		System.out.println(rect);
	}

	private void drawGraph(Graphics2D g) {
		CmnDate to = new CmnDate();
		CmnDate fr = new CmnDate();
		fr.add(CmnDate.MINUTE, -30);

		RateHist cur, pre = null;
		double dMax = 0, dMin = 999999999999999.0;
        int     nPos;
        for ( nPos = 0; nPos < vecHist.size(); nPos++ ) {
            cur = (RateHist )vecHist.get( nPos );
            if ( cur.dt.after( fr )) {
                break;
            }
        }
        for ( int i = 0; i < nPos-1; i++ ) {
            if ( vecHist.size() > 1 ) {
                cur = (RateHist )vecHist.remove( 0 );
            }
        }
        for ( int i = vecHist.size() - 1; i >= 0; i-- ) {
            cur = (RateHist )vecHist.get( i );
            double rate = cur.rate.doubleValue();
            if ( dMax < rate ) {
                dMax = rate;
            }
            if ( dMin > rate ) {
                dMin = rate;
            }
        }
        double  dDiv = Math.pow( 10, point - 1 );
        dMin = ( Math.floor( dMin*dDiv )) / dDiv ;
        dMax = ( Math.floor( dMax*dDiv ) + 1.0 ) / dDiv ;

        Graph2D grf = new Graph2D( fr, to, new Double( dMin ), new Double( dMax ));
        StairGraph dat = new StairGraph( fr, to,
                        new BigDecimal( dMin ), new BigDecimal( dMax ));
        dat.setStroke(
            new BasicStroke( 1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL ));
        DefaultStorage dt = (DefaultStorage)dat.getStorage();
        grf.addSeries( dat );
        dat.setLineColor( Color.black );

       Font font = new Font( "Dialog", Font.PLAIN, 10 );
        XScale  xs = (XScale )grf.getXScale();
        xs.add( fr, fr.format( "HH:mm" ));
        CmnDate tmp = (CmnDate )fr.clone();
        tmp.set( CmnDate.SECOND, 0 );
        for ( ;; ) {
            tmp.add( CmnDate.MINUTE, 1 );
            if ( tmp.after( to ) ) {
                break;
            }
            if (( tmp.get( CmnDate.MINUTE ) % 10 ) == 0 ) {
                double dFr = fr.getTime().getTime();
                double dTo = to.getTime().getTime();
                double dCr = tmp.getTime().getTime();
                if (( ( dCr - dFr ) < ( 3 * 60 * 1000 )) ||
                    ( ( dTo - dCr ) < ( 3 * 60 * 1000 )))
                {
                    xs.add( tmp.clone(), "" );
                } else {
                    xs.add( tmp.clone(), tmp.format( "HH:mm" ));
                }
            }
        }
        xs.add( to, to.format( "HH:mm" ));
        xs.setStroke( new BasicStroke( 1,
                BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND,
                0, new float[]{2,5}, 0 ));
        xs.setScaleLine( true );
        xs.setFont( font );

        YScale  ys = (YScale )grf.getYScale(); //new YScale( dMin, dMax );
        ys.setFont( font );
        ys.add( dMin, df.format( dMin ));
        ys.add( dMax, df.format( dMax ));
        for ( int i = (int )(dMin*dDiv); i < (int )(dMax*dDiv); i++ ) {
            ys.add( (double)i / dDiv, df.format( (double)i / dDiv ));
        }
        ys.setStroke( new BasicStroke( 1,
                BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND,
                0, new float[]{2,5}, 0 ));
        ys.setScaleLine( true );

        cur = null;
        for ( int i = 0; i < vecHist.size(); i++ ) {
            cur = (RateHist )vecHist.get( i );
            dt.add( new CmnDate( cur.dt.getTime()), cur.rate );
            pre = cur;
        }
        if ( cur != null ) {
            dt.add( new CmnDate(), cur.rate );
        }
/*
        Rectangle rect = new Rectangle( 50, 10, 250, 100 );
*/
        Rectangle rect = new Rectangle(40, 40, getWidth()-60, getHeight()-60);
        grf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                RenderingHints.VALUE_ANTIALIAS_ON);
        grf.drawGraph( g, rect );
	}

	public void paint(Graphics g) {
		super.paint(g);
		drawGraph((Graphics2D)g);
	}

	public void received(ReceiveEvent e) {
		FxRate newR = (FxRate )e.getData();
		if ( !newR.getTerm().equals( term )) {
			return;
		}
		RateHist hist = new RateHist();
		hist.dt = new CmnDate();
		hist.rate = newR.getBid();
		vecHist.add(hist);
		repaint();
	}

	public void statusChanged( StatusChangeEvent e ) {
	}

	public void closed() {
		try {
			recv.removeReceiveListener( key, this );
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
